package com.smppw.analysis.application.service.info;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.domain.dataobject.PrivatelyFundBaseInfoDo;
import com.smppw.analysis.domain.dataobject.PubliclyFundBaseInfoDo;
import com.smppw.analysis.domain.dto.info.PrivatelyFundBaseInfoVO;
import com.smppw.analysis.domain.dto.info.PubliclyFundBaseInfoVO;
import com.smppw.analysis.domain.service.BaseInfoService;
import com.smppw.utils.StrategyHandleUtils;
import org.springframework.stereotype.Service;

import java.util.Map;

@Service
public class FundInfoService {
    private static final Map<String, String> ACCRUED_FREQUENCY = MapUtil.newHashMap(5);
    private static final Map<String, String> ACCRUED_METHOD = MapUtil.newHashMap(3);
    private static final Map<Integer, String> EDUCATION_NAME_MAP = MapUtil.newHashMap(8);
    private static final Map<Integer, String> FUND_STATUS_MAP = MapUtil.newHashMap(11);
    private static final Map<Integer, String> FUND_TYPE_MAP = MapUtil.newHashMap(11);
    private static final Map<Integer, String> BASE_CURRENCY_MAP = MapUtil.newHashMap(5);
    private static final Map<Integer, String> APPLYING_MARKET_MAP = MapUtil.newHashMap(3);

    static {
        ACCRUED_FREQUENCY.put("1", "月");
        ACCRUED_FREQUENCY.put("2", "季");
        ACCRUED_FREQUENCY.put("3", "半年");
        ACCRUED_FREQUENCY.put("4", "年");
        ACCRUED_FREQUENCY.put("0", "不固定");
        ACCRUED_METHOD.put("1", "高水位");
        ACCRUED_METHOD.put("2", "高水位+赎回时补充");
        ACCRUED_METHOD.put("3", "单客户高水位");

        EDUCATION_NAME_MAP.put(1, "小学");
        EDUCATION_NAME_MAP.put(2, "中学");
        EDUCATION_NAME_MAP.put(3, "大专");
        EDUCATION_NAME_MAP.put(4, "本科");
        EDUCATION_NAME_MAP.put(5, "硕士");
        EDUCATION_NAME_MAP.put(6, "博士");
        EDUCATION_NAME_MAP.put(7, "博士后");
        EDUCATION_NAME_MAP.put(-1, "其他");
        BASE_CURRENCY_MAP.put(1, "人民币");
        BASE_CURRENCY_MAP.put(2, "港币");
        BASE_CURRENCY_MAP.put(3, "美元");
        BASE_CURRENCY_MAP.put(4, "份");
        BASE_CURRENCY_MAP.put(-1, "其他");

        FUND_STATUS_MAP.put(1, "募集中");
        FUND_STATUS_MAP.put(2, "开放运行");
        FUND_STATUS_MAP.put(4, "提前清算");
        FUND_STATUS_MAP.put(5, "到期清算");
        FUND_STATUS_MAP.put(6, "发行失败");
        FUND_STATUS_MAP.put(7, "投顾协议已终止");
        FUND_STATUS_MAP.put(8, "延期清算");
        FUND_STATUS_MAP.put(9, "非正常清算");
        FUND_STATUS_MAP.put(10, "已终止");
        FUND_STATUS_MAP.put(11, "已作废 ");
        FUND_STATUS_MAP.put(-1, "其他");

        FUND_TYPE_MAP.put(1, "信托计划");
        FUND_TYPE_MAP.put(3, "券商资管");
        FUND_TYPE_MAP.put(4, "公募专户");
        FUND_TYPE_MAP.put(5, "单账户");
        FUND_TYPE_MAP.put(6, "证券投资基金");
        FUND_TYPE_MAP.put(7, "海外基金");
        FUND_TYPE_MAP.put(8, "期货资管");
        FUND_TYPE_MAP.put(9, "保险资管");
        FUND_TYPE_MAP.put(10, "创业投资基金");
        FUND_TYPE_MAP.put(11, "股权投资基金 ");
        FUND_TYPE_MAP.put(12, "银行理财 ");
        FUND_TYPE_MAP.put(14, "私募资产配置基金 ");
        FUND_TYPE_MAP.put(15, "公募基金 ");
        FUND_TYPE_MAP.put(-1, "其他");

        APPLYING_MARKET_MAP.put(1, "场内");
        APPLYING_MARKET_MAP.put(2, "场外");
        APPLYING_MARKET_MAP.put(3, "场内和场外");

    }

    private final BaseInfoService baseInfoService;

    public FundInfoService(BaseInfoService baseInfoService) {
        this.baseInfoService = baseInfoService;
    }

    public PrivatelyFundBaseInfoVO getFundBaseInfo(String fundId) {
        PrivatelyFundBaseInfoVO privatelyFundBaseInfoVO = new PrivatelyFundBaseInfoVO();
        //私募基金基本信息
        PrivatelyFundBaseInfoDo fundInfo = this.baseInfoService.listPrivatelyFundInfo(fundId);
        if (fundInfo == null) {
            return privatelyFundBaseInfoVO;
        }
        privatelyFundBaseInfoVO = BeanUtil.copyProperties(fundInfo, PrivatelyFundBaseInfoVO.class);
        privatelyFundBaseInfoVO.setBaseCurrency(fundInfo.getBaseCurrency() != null ? BASE_CURRENCY_MAP.get(Integer.valueOf(fundInfo.getBaseCurrency())) : null);
        privatelyFundBaseInfoVO.setUmbrellaFund(fundInfo.getUmbrellaFund() == 1 ? "是" : "否");
        privatelyFundBaseInfoVO.setIstiered(fundInfo.getIstiered() == 1 ? "是" : "否");
        privatelyFundBaseInfoVO.setFundTypeValue(fundInfo.getFundType() != null ? FUND_TYPE_MAP.get(Integer.valueOf(fundInfo.getFundType())) : null);
//        //获取基金的基金经理
//        List<FundManagerInfoDo> fundManagerInfoList = this.baseInfoService.listFundManagerByFundId(fundId);
//        if (CollUtil.isNotEmpty(fundManagerInfoList)) {
//            privatelyFundBaseInfoVO.setFundManager(fundManagerInfoList.stream()
//                    .map(FundManagerInfoDo::getManagerName).collect(Collectors.joining(",")));
//        }
        //处理投资顾问,基金管理人,基金托管人
//        List<String> companyIdList = CollUtil.newArrayList();
//        List<CompanyInformationDo> companyInformationDos;
//        String trustId = fundInfo.getTrustId();
//        if (StrUtil.isNotBlank(trustId)) {
//            companyIdList.add(trustId);
//        }
//        String custodianId = fundInfo.getCustodianId();
//        if (StrUtil.isNotBlank(custodianId)) {
//            companyIdList.add(custodianId);
//        }
//        String advisorId = fundInfo.getAdvisorId();
//        if (StrUtil.isNotBlank(advisorId)) {
//            companyIdList.add(advisorId);
//        }
//        if (CollUtil.isNotEmpty(companyIdList)) {
//            companyInformationDos = companyInformationDao.listCompany(companyIdList);
//            if (CollUtil.isNotEmpty(companyInformationDos)) {
//                Map<String, CompanyInformationDo> companyInfoMap = companyInformationDos.stream().collect(Collectors.toMap(k -> k.getCompanyId(), v -> v));
//                privatelyFundBaseInfoVO.setTrustName(companyInfoMap.get(trustId) != null ? companyInfoMap.get(trustId).getCompanyShortName() : null);
//                privatelyFundBaseInfoVO.setTrustFullName(companyInfoMap.get(trustId) != null ? companyInfoMap.get(trustId).getCompanyName() : null);
//                privatelyFundBaseInfoVO.setCustodianName(companyInfoMap.get(custodianId) != null ? companyInfoMap.get(custodianId).getCompanyName() : null);
//                privatelyFundBaseInfoVO.setAdvisorName(companyInfoMap.get(advisorId) != null ? companyInfoMap.get(advisorId).getCompanyShortName() : null);
//            }
//        }
        //处理基金策略
        Integer strategy = fundInfo.getStrategy();
        Integer subStrategy = fundInfo.getSubstrategy();
        Integer thirdStrategy = fundInfo.getThirdStrategy();
        privatelyFundBaseInfoVO.setStrategy(StrategyHandleUtils.getStrategyById(strategy).getStrategyNameDesc());
        privatelyFundBaseInfoVO.setSubstrategy(StrategyHandleUtils.getStrategyById(subStrategy).getStrategyNameDesc());
        privatelyFundBaseInfoVO.setThirdStrategy(StrategyHandleUtils.getStrategyById(thirdStrategy).getStrategyNameDesc());
        return privatelyFundBaseInfoVO;
    }

    public PubliclyFundBaseInfoVO getMfFundBaseInfo(String fundId) {
        PubliclyFundBaseInfoVO publiclyFundBaseInfoVO = new PubliclyFundBaseInfoVO();
        PubliclyFundBaseInfoDo fundInfo = this.baseInfoService.listPublicFundInfo(fundId);
        if (fundInfo == null) {
            return publiclyFundBaseInfoVO;
        }
        publiclyFundBaseInfoVO = BeanUtil.copyProperties(fundInfo, PubliclyFundBaseInfoVO.class);
        publiclyFundBaseInfoVO.setBaseCurrency(fundInfo.getBaseCurrency() != null ? BASE_CURRENCY_MAP.get(Integer.valueOf(fundInfo.getBaseCurrency())) : null);
        publiclyFundBaseInfoVO.setFundStatus(fundInfo.getFundStatusId() != null ? FUND_STATUS_MAP.get(fundInfo.getFundStatusId()) : null);
//        //获取基金的基金经理
//        List<FundManagerInfoDo> fundManagerInfoList = this.baseInfoService.listFundManagerByFundId(fundId);
//        if (CollUtil.isNotEmpty(fundManagerInfoList)) {
//            List<ValueLabelVO> managerIdNameList = fundManagerInfoList.stream().map(e -> {
//                ValueLabelVO valueLabelVO = new ValueLabelVO();
//                valueLabelVO.setValue(e.getManagerId());
//                valueLabelVO.setLabel(e.getManagerName());
//                return valueLabelVO;
//            }).collect(Collectors.toList());
//            publiclyFundBaseInfoVO.setManagers(managerIdNameList);
//        }
        //处理基金策略
        Integer strategy = fundInfo.getStrategy();
        Integer subStrategy = fundInfo.getSubstrategy();
        publiclyFundBaseInfoVO.setStrategy(StrategyHandleUtils.getStrategyById(strategy).getStrategyNameDesc());
        publiclyFundBaseInfoVO.setSubstrategy(StrategyHandleUtils.getStrategyById(subStrategy).getStrategyNameDesc());
//        //基金管理人,基金托管人
//        List<String> companyIdList = CollUtil.newArrayList();
//        List<CompanyInformationDo> companyInformationDos;
//        String trustId = fundInfo.getTrustId();
//        if (StrUtil.isNotBlank(trustId)) {
//            companyIdList.add(trustId);
//        }
//        String custodianId = fundInfo.getCustodianId();
//        if (StrUtil.isNotBlank(custodianId)) {
//            companyIdList.add(custodianId);
//        }
//        if (CollUtil.isNotEmpty(companyIdList)) {
//            companyInformationDos = companyInformationDao.listCompany(companyIdList);
//            if (CollUtil.isNotEmpty(companyInformationDos)) {
//                Map<String, CompanyInformationDo> companyInfoMap = companyInformationDos.stream().collect(Collectors.toMap(k -> k.getCompanyId(), v -> v));
//                publiclyFundBaseInfoVO.setTrustName(companyInfoMap.get(trustId) != null ? companyInfoMap.get(trustId).getCompanyShortName() : null);
//                publiclyFundBaseInfoVO.setCustodianName(companyInfoMap.get(custodianId) != null ? companyInfoMap.get(custodianId).getCompanyName() : null);
//            }
//        }
//        //开放日和申赎状态
//        handleOpenDayAndRedeemType(publiclyFundBaseInfoVO, fundId);
//        handleFundArchivesInfo(publiclyFundBaseInfoVO, fundId);
//        //初始规模和最新规模
//        handleFundScale(publiclyFundBaseInfoVO, fundId);
        return publiclyFundBaseInfoVO;
    }
}
