package com.smppw.analysis.client;

import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.infrastructure.components.RequestWrapper;
import com.smppw.analysis.infrastructure.config.AnalysisProperty;
import com.smppw.analysis.infrastructure.utils.CommonUtils;
import com.smppw.common.pojo.ResultVo;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/v1/")
public class TestApi {
    private final AnalysisProperty property;

    public TestApi(AnalysisProperty property) {
        this.property = property;
    }

    /**
     * 生成签名，测试用的
     *
     * @param request 请求参数，包含appKey
     * @return /
     */
    @PostMapping("sign")
    public ResultVo<String> generateSign(HttpServletRequest request) throws Exception {
        String body = new RequestWrapper(request).getBodyString();
        Map<String, Object> params = CommonUtils.str2Map(body);
        String appKey = MapUtil.getStr(params, "appKey");
        params.entrySet().removeIf(next -> "appKey".equalsIgnoreCase(next.getKey()));
        List<AnalysisProperty.AppSign> appSigns = this.property.getAppSigns();
        String secret = appSigns.stream().filter(e -> appKey.equalsIgnoreCase(e.getAppKey()))
                .map(AnalysisProperty.AppSign::getAppSecret).findFirst().orElse(null);
        String sign = CommonUtils.createSign(params, secret);
        return ResultVo.ok(sign);
    }
}
