package com.smppw.analysis.domain.dao.info;

import cn.hutool.core.collection.ListUtil;
import com.smppw.analysis.domain.dataobject.info.BaseInfoDO;
import com.smppw.common.pojo.enums.Frequency;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangzaijun
 * @date 2023/8/11 18:52
 * @description 基本信息业务逻辑处理，参考净值的（也考虑缓存）
 */
public interface InfoDao<R extends BaseInfoDO> {
    /**
     * 获取单个标的的基本信息
     *
     * @param secId 标的id
     * @return /
     */
    default R getInfo(String secId) {
        List<R> infos = this.listMultiInfo(ListUtil.toList(secId));
        Map<String, R> secInfoMap = infos.stream().collect(Collectors.toMap(BaseInfoDO::getRefId, e -> e));
        return secInfoMap.get(secId);
    }

    /**
     * 获取多个标的的id与名称对应关系
     *
     * @param secIds 标的id集合
     * @return /
     */
    default Map<String, String> getSecNameMap(List<String> secIds) {
        List<R> infos = this.listMultiInfo(secIds);
        return infos.stream().filter(e -> e.getRefName() != null)
                .collect(Collectors.toMap(BaseInfoDO::getRefId, BaseInfoDO::getRefName));
    }

    /**
     * 获取多个标的的id与净值频率对应关系
     *
     * @param secIds 标的id集合
     * @return /
     */
    default Map<String, Frequency> getSecFreqMap(List<String> secIds) {
        List<R> infos = this.listMultiInfo(secIds);
        return infos.stream().filter(e -> e.getFrequency() != null)
                .collect(Collectors.toMap(BaseInfoDO::getRefId, BaseInfoDO::getFrequency));
    }

    /**
     * 获取多各标的id与基本信息的对应关系
     *
     * @param secIds 标的id集合
     * @return /
     */
    default Map<String, R> getSecInfoMap(List<String> secIds) {
        List<R> infos = this.listMultiInfo(secIds);
        return infos.stream().collect(Collectors.toMap(BaseInfoDO::getRefId, e -> e));
    }

    /**
     * 获取多个同类型标的的基本信息
     *
     * @param secIds 标的id集合
     * @return /
     */
    List<R> listMultiInfo(List<String> secIds);
}
