package com.smppw.analysis.domain.manager.info;

import com.smppw.analysis.domain.dao.info.InfoFactory;
import com.smppw.analysis.domain.dao.nav.NavFactory;
import com.smppw.analysis.domain.dataobject.info.BaseInfoDO;
import com.smppw.analysis.domain.dto.info.HeadInfoParams;
import com.smppw.analysis.domain.dto.info.HeadInfoVO;
import com.smppw.analysis.infrastructure.exception.APIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

/**
 * @author wangzaijun
 * @date 2023/8/11 13:49
 * @description 所有标的头部信息获取（获取净值已经抽象、获取基本信息可以抽象）
 */
public abstract class AbstractHeadInfo<R extends HeadInfoVO, DO extends BaseInfoDO> implements HeadInfo<R> {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    protected final NavFactory navFactory;
    protected final InfoFactory infoFactory;

    public AbstractHeadInfo(NavFactory navFactory, InfoFactory infoFactory) {
        this.navFactory = navFactory;
        this.infoFactory = infoFactory;
    }

    @Override
    public R get(HeadInfoParams params) {
        String fundId = params.getSecId();
        DO infoDO = this.loadInfoDO(fundId);
        if (infoDO == null) {
            throw new APIException("基金不存在");
        }
        Map<String, String> secNameMap = this.getOtherSecNameMap(infoDO);
        R result = this.handleBaseInfo(infoDO, secNameMap);
        this.handleLastNav(result, params.getSecType());
        this.handleStrategySummaryName(result);
        return result;
    }

    protected abstract DO loadInfoDO(String secId);

    protected abstract R handleBaseInfo(DO infoDO, Map<String, String> secNameMap);

    /**
     * 获取出当前基金外所有标的的名称映射
     *
     * @param infoDO /
     * @return /
     */
    protected abstract Map<String, String> getOtherSecNameMap(DO infoDO);
//    {
//        List<String> secIds = ListUtil.list(false);
////        secIds.add(fundInfo.getTrustId());
//        secIds.add(infoDO.getPrimaryBenchmarkId());
//        if (infoDO.getStrategy() != null) {
//            secIds.add(infoDO.getStrategy() + "");
//        }
//        if (infoDO.getSubstrategy() != null) {
//            secIds.add(infoDO.getSubstrategy() + "");
//        }
//        if (infoDO.getFirstStrategy() != null) {
//            secIds.add(infoDO.getFirstStrategy() + "");
//        }
//        if (infoDO.getSecondStrategy() != null) {
//            secIds.add(infoDO.getSecondStrategy() + "");
//        }
//        if (infoDO.getThirdStrategy() != null) {
//            secIds.add(infoDO.getThirdStrategy() + "");
//        }
////        if (StrUtil.isNotBlank(fundInfo.getManagerIds())) {
////            List<String> tempList = StrUtil.split(fundInfo.getManagerIds(), ",");
////            if (CollUtil.isNotEmpty(tempList)) {
////                secIds.addAll(tempList);
////            }
////        }
//        return this.baseInfoService.querySecName(secIds);
//    }

    /**
     * 最新的净值数据
     *
     * @param headInfo 基本信息
     * @param secType  标的类型
     */
    protected abstract void handleLastNav(R headInfo, String secType);
//    {
//        String fundId = headInfo.getRefId();
//        NavDao instance = this.navFactory.getInstance(secType);
//        Map<String, NavDto> navMap = instance.getLastNav(ListUtil.of(fundId));
//        NavDto navDto = navMap.getOrDefault(fundId, new NavDto());
//        headInfo.setPriceDate(navDto.getPriceDate());
//        headInfo.setOriginalNav(StrUtil.toStringOrNull(navDto.getOriginalNav()));
//        headInfo.setCumulativeNav(StrUtil.toStringOrNull(navDto.getCumulativeNav()));
//        headInfo.setCumulativeNavWithdrawal(StrUtil.toStringOrNull(navDto.getCumulativeNavWithdrawal()));
//    }

//    /**
//     * 基金经理id和名称
//     *
//     * @param headInfo   /
//     * @param secNameMap /
//     * @param managerIds /
//     */
//    private void handleManager(FundHeadInfoVO headInfo, Map<String, String> secNameMap, String managerIds) {
//        if (StrUtil.isBlank(managerIds)) {
//            return;
//        }
//        List<ValueLabelVO> managers = ListUtil.listMultiInfo(true);
//        List<String> managerList = StrUtil.split(managerIds, ",");
//        for (String managerId : managerList) {
//            managers.add(new ValueLabelVO(managerId, secNameMap.get(managerId)));
//        }
//        headInfo.setManagers(managers);
//    }

//    /**
//     * 基本信息填充
//     *
//     * @param headInfo   /
//     * @param infoDO     /
//     * @param secNameMap /
//     */
//    protected abstract void handleBaseInfo(R headInfo, DO infoDO, Map<String, String> secNameMap);
//    {
////        String trustId = headInfo.getTrustId();
//        headInfo.setRefShortName(infoDO.getFundShortName());
//        headInfo.setRefName(infoDO.getFundName());
//        headInfo.setRegisterNumber(infoDO.getRegisterNumber());
//        headInfo.setAmacUrl(infoDO.getAmacUrl());
//        headInfo.setRaiseType(infoDO.getRaiseType() == 1 ? "私募" : "公募");
//        RaiseType raiseType = RaiseType.getRaiseType(infoDO.getRaiseType());
//        headInfo.setRaiseTypeValue(raiseType.name());
//        headInfo.setInceptionDate(infoDO.getInceptionDate());
////        headInfo.setTrustName(secNameMap.get(trustId));
//        headInfo.setFundStatus(infoDO.getFundStatus() == null ? null : infoDO.getFundStatus().toString());
////        headInfo.setCompanyAssetSize(this.fundInformationDao.getLastCompanyScale(trustId)); // 公司规模
////        headInfo.setFundScale(this.fundInformationDao.getFundLastAssetSize(headInfo.getRefId())); // 基金规模
//        headInfo.setPrimaryBenchmarkName(secNameMap.get(headInfo.getPrimaryBenchmarkId()));
//    }

    /**
     * 策略名称拼接处理
     *
     * @param info /
     */
    protected abstract void handleStrategySummaryName(R info);
//    {
//        Integer thirdStrategyId = info.getThirdStrategyId();
//        if (thirdStrategyId == null || Objects.equals(-1, thirdStrategyId)) {
//            info.setThirdStrategyId(null);
//            if (StrUtil.isBlank(info.getSubstrategy())) {
//                info.setStrategySummaryName(info.getStrategy());
//            } else {
//                info.setStrategySummaryName(info.getStrategy() + "/" + info.getSubstrategy());
//            }
//        } else {
//            info.setStrategySummaryName(info.getSubstrategy() + "/" + info.getThirdStrategy());
//        }
//    }
}
