package com.smppw.analysis.domain.manager.info.handler;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dao.info.InfoFactory;
import com.smppw.analysis.domain.dao.nav.NavDao;
import com.smppw.analysis.domain.dao.nav.NavFactory;
import com.smppw.analysis.domain.dataobject.info.PrivateFundInfoDO;
import com.smppw.analysis.domain.dto.info.PrivateFundHeadInfoVO;
import com.smppw.analysis.domain.manager.info.AbstractHeadInfo;
import com.smppw.analysis.domain.manager.info.HeadInfoConstants;
import com.smppw.common.pojo.dto.NavDto;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.util.Map;

@Component(HeadInfoConstants.PRIVATE_FUND)
public class PrivateHeadInfo extends AbstractHeadInfo<PrivateFundHeadInfoVO, PrivateFundInfoDO> {
    public PrivateHeadInfo(NavFactory navFactory, InfoFactory infoFactory) {
        super(navFactory, infoFactory);
    }

    @Override
    protected PrivateFundInfoDO loadInfoDO(String secId) {
        String secType = SecType.getSecType(secId, null);
        return this.infoFactory.<PrivateFundInfoDO>getInstance(secType).getInfo(secId);
    }

    @Override
    protected PrivateFundHeadInfoVO handleBaseInfo(PrivateFundInfoDO infoDO, Map<String, String> secNameMap) {
        PrivateFundHeadInfoVO infoVo = new PrivateFundHeadInfoVO();
        infoVo.setRefId(infoDO.getRefId());
        infoVo.setDetailId(infoDO.getRefId());
        infoVo.setRefName(infoDO.getRefName());
        infoVo.setRefShortName(infoDO.getFundShortName());
        infoVo.setInceptionDate(infoDO.getInceptionDate());
        infoVo.setRegisterNumber(infoDO.getRegisterNumber());
        infoVo.setFrequency(infoDO.getFrequency());
        return infoVo;
    }

    @Override
    protected Map<String, String> getOtherSecNameMap(PrivateFundInfoDO infoDO) {
        return MapUtil.empty();
    }

    @Override
    protected void handleLastNav(PrivateFundHeadInfoVO headInfo, String secType) {
        String fundId = headInfo.getRefId();
        NavDao instance = this.navFactory.getInstance(secType);
        Map<String, NavDto> navMap = instance.getLastNav(ListUtil.of(fundId));
        NavDto navDto = navMap.getOrDefault(fundId, new NavDto());
        headInfo.setPriceDate(navDto.getPriceDate());
        headInfo.setOriginalNav(StrUtil.toStringOrNull(navDto.getOriginalNav()));
        headInfo.setCumulativeNav(StrUtil.toStringOrNull(navDto.getCumulativeNav()));
        headInfo.setCumulativeNavWithdrawal(StrUtil.toStringOrNull(navDto.getCumulativeNavWithdrawal()));
    }

    @Override
    protected void handleStrategySummaryName(PrivateFundHeadInfoVO info) {

    }
}
