package com.smppw.analysis.infrastructure.utils;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.DigestUtils;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CommonUtils {
    private static final Logger logger = LoggerFactory.getLogger(CommonUtils.class);

    /**
     * 一个简单的数据转换，object对象转list
     *
     * @param obj   待转换对象
     * @param clazz 泛型类型
     * @param <T>   类型参数
     * @return 对应泛型类型的实体的集合
     */
    public static <T> List<T> convertList(Object obj, Class<T> clazz) {
        try {
            return JSONUtil.toList(JSONUtil.toJsonStr(obj), clazz);
        } catch (Exception e) {
            logger.warn(StrUtil.format("数据转换list失败：{}", e.getMessage()));
        }
        return ListUtil.empty();
    }

    /**
     * 一个简单的数据转换，object对象转bean
     *
     * @param obj   待转换对象
     * @param clazz 泛型类型
     * @param <T>   类型参数
     * @return 对应泛型类型实体
     */
    public static <T> T convertBean(Object obj, Class<T> clazz) {
        try {
            return JSONUtil.toBean(JSONUtil.toJsonStr(obj), clazz);
        } catch (Exception e) {
            logger.warn(StrUtil.format("数据转换实体失败：{}", e.getMessage()));
        }
        return null;
    }

    /**
     * json字符串转Map
     */
    public static Map<String, Object> str2Map(String str) {
        /*先转换为JSON对象*/
        JSONObject entries = JSONUtil.parseObj(str);
        Map<String, Object> resMap = new HashMap<>();
        for (Map.Entry<String, Object> entry : entries.entrySet()) {
            resMap.put(entry.getKey(), entry.getValue());
        }
        return resMap;
    }

    public static String createSign(Map<String, Object> params, String... others) throws Exception {
        StringBuilder ketStr = new StringBuilder();
        Object[] keys = params.keySet().toArray();
        Arrays.sort(keys);
        for (Object key : keys) {
            String valueStr = JSONUtil.toJsonStr(params.get(key.toString()));
            ketStr.append(key)
                    .append("=")
                    .append(valueStr)
                    .append(",");
        }
        if (others != null && others.length > 0) {
            for (String other : others) {
                ketStr.append(other);
            }
        }
        return DigestUtils.md5DigestAsHex(ketStr.toString().getBytes(StandardCharsets.UTF_8));
    }

    public static void main(String[] args) throws Exception {
        String appSecret = "abcdef";
        Map<String, Object> params = new HashMap<>();
        params.put("secId", "MF00005JMD");
        params.put("indexIds", ListUtil.of("IN00000008", "IN0000000M"));
        String sign = createSign(params, appSecret);
        System.out.println(sign);
    }
}
